# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-04-13 13:26
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('openstack', '0034_remove_start_time'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='ipmapping',
            name='project',
        ),
        migrations.DeleteModel(
            name='IpMapping',
        ),
    ]
