# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-09-05 13:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0026_remove_type_protect_template'),
    ]

    operations = [
        migrations.AlterField(
            model_name='offeringtemplate',
            name='name',
            field=models.CharField(max_length=150, verbose_name='name'),
        ),
    ]
