# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-26 17:12
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_jira', '0004_issue_types'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='issue',
            name='impact',
        ),
        migrations.RemoveField(
            model_name='project',
            name='impact_field',
        ),
    ]
