# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-03-15 13:37
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0064_remove_resource_count_project_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='resource',
            name='cost',
            field=models.DecimalField(blank=True, decimal_places=10, max_digits=22, null=True),
        ),
        migrations.AlterField(
            model_name='orderitem',
            name='limits',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict),
        ),
    ]
