# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-04-12 06:55
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openstack', '0040_unique_floating_ip'),
    ]

    operations = [
        migrations.AddField(
            model_name='tenant',
            name='default_volume_type_name',
            field=models.CharField(blank=True, help_text='Volume type name to use when creating volumes.', max_length=100),
        ),
    ]
