# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-09-14 11:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0034_plan_quota'),
    ]

    operations = [
        migrations.AddField(
            model_name='plancomponent',
            name='description',
            field=models.CharField(blank=True, max_length=500, verbose_name='description'),
        ),
        migrations.AddField(
            model_name='plancomponent',
            name='measured_unit',
            field=models.CharField(default='', help_text='Unit of measurement, for example, GB.', max_length=30, blank=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='plancomponent',
            name='name',
            field=models.CharField(default='', help_text='Display name for the measured unit, for example, Floating IP.', max_length=150),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='plancomponent',
            name='type',
            field=models.CharField(help_text='Unique internal name of the measured unit, for example floating_ip.', max_length=50),
        ),
    ]
