# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-04 09:02
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations
import django_fsm
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0011_order_item'),
    ]

    operations = [
        migrations.AlterField(
            model_name='offering',
            name='attributes',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict,
                                                                 help_text='Fields describing Category.'),
        ),
    ]
