# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-03-22 14:51
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0054_payment_details'),
        ('contenttypes', '0002_remove_content_type_name'),
        ('python_management', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='pythonmanagement',
            name='instance_content_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='contenttypes.ContentType'),
        ),
        migrations.AddField(
            model_name='pythonmanagement',
            name='instance_object_id',
            field=models.PositiveIntegerField(null=True),
        ),
        migrations.AddField(
            model_name='pythonmanagement',
            name='project',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='structure.Project'),
        ),
        migrations.AddField(
            model_name='pythonmanagement',
            name='system_user',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='virtualenvironment',
            name='jupyter_hub_global',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterUniqueTogether(
            name='pythonmanagement',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='pythonmanagement',
            name='instance',
        ),
        migrations.RemoveField(
            model_name='pythonmanagement',
            name='service_project_link',
        ),
    ]
