{% extends "admin/change_form.html" %}
{% load i18n %}

{% block submit_buttons_bottom %}
  <div class="submit-row">
    <a href="../../">{% blocktrans %}Back to list{% endblocktrans %}</a>
  </div>
{% endblock %}
