{% extends "admin/change_form.html" %}
{% load i18n admin_urls %}

{% block object-tools-items %}
<li>
    {% url opts|admin_urlname:'history' original.pk|admin_urlquote as history_url %}
    <a href="{% add_preserved_filters history_url %}" class="historylink">{% trans "History" %}</a>
</li>
{% for link in extra_object_links %}
    {% if link.show %}
        <li>
            <a href="{{ link.href }}" class="grp-state-focus" class="addlink">{{ link.label }}</a>
        </li>
    {% endif %}
{% endfor %}
{% if has_absolute_url %}<li><a href="{{ absolute_url }}" class="viewsitelink">{% trans "View on site" %}</a></li>{% endif %}
{% endblock %}
