# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-10-31 08:19
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0029_issue_resolution_date'),
    ]

    operations = [
        migrations.AddField(
            model_name='issue',
            name='template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='issues', to='support.Template'),
        ),
    ]
