# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-05 13:07
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('support_invoices', '0001_initial'),
    ]

    operations = [
        migrations.DeleteModel(
            name='RequestBasedOffering',
        ),
    ]
