# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-01-17 13:53
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0050_add_component_limit'),
    ]

    operations = [
        migrations.AddField(
            model_name='resource',
            name='backend_metadata',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict),
        ),
    ]
