# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-03-25 16:19
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0066_cartitem_cost'),
    ]

    operations = [
        migrations.AddField(
            model_name='offeringcomponent',
            name='disable_quotas',
            field=models.BooleanField(default=False, help_text='Do not allow user to specify quotas when offering is provisioned.'),
        ),
    ]
