# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-04-10 16:46
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0025_copy_certifications_from_existing_settings'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='backup',
            name='start_time',
        ),
        migrations.RemoveField(
            model_name='backupschedule',
            name='start_time',
        ),
        migrations.RemoveField(
            model_name='snapshot',
            name='start_time',
        ),
        migrations.RemoveField(
            model_name='snapshotschedule',
            name='start_time',
        ),
        migrations.RemoveField(
            model_name='volume',
            name='start_time',
        ),
    ]
