# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-06-21 21:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('invoices', '0013_set_projects_for_invoice_items'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentdetails',
            name='accounting_start_date',
            field=models.DateTimeField('Start date of accounting', default=django.utils.timezone.now),
        ),
    ]
