# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-25 11:43
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0078_fix_plan_component_amount'),
    ]

    operations = [
        migrations.AddField(
            model_name='orderitem',
            name='old_plan',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='marketplace.Plan'),
        ),
    ]
