# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-03-26 08:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0071_componentusage_uuid_unique'),
    ]

    operations = [
        migrations.AddField(
            model_name='offeringcomponent',
            name='article_code',
            field=models.CharField(blank=True, max_length=30),
        ),
        migrations.AddField(
            model_name='offeringcomponent',
            name='product_code',
            field=models.CharField(blank=True, max_length=30),
        ),
    ]
