# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-09-11 19:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0032_orderitem_error_message'),
    ]

    operations = [
        migrations.AddField(
            model_name='plancomponent',
            name='billing_type',
            field=models.CharField(choices=[('fixed', 'Fixed-price'), ('usage', 'Usage-based')], default='fixed', max_length=5),
        ),
    ]
