# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-06-07 08:52
from __future__ import unicode_literals

from django.db import migrations, models
import waldur_core.core.validators


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0051_add_customer_email_phone_agreement_number'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='access_subnets',
            field=models.TextField(blank=True, default='', validators=[waldur_core.core.validators.validate_cidr_list], help_text='Enter a comma separated list of IPv4 or IPv6 CIDR addresses from where connection to self-service is allowed.'),
        ),
    ]
