# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-25 09:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0056_plan_max_amount'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='backend_id',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]
