# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-12-28 12:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0049_offering_billable'),
    ]

    operations = [
        migrations.AddField(
            model_name='offeringcomponent',
            name='limit_amount',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='offeringcomponent',
            name='limit_period',
            field=models.CharField(blank=True, choices=[('month', 'Maximum monthly - every month service provider can report up to the amount requested by user.'), ('total', 'Maximum total - SP can report up to the requested amount over the whole active state of resource.')], max_length=5, null=True),
        ),
    ]
