# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-03-12 11:26
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('marketplace', '0063_add_scope_for_resource_count'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='aggregateresourcecount',
            unique_together=set([('category', 'content_type', 'object_id')]),
        ),
        migrations.RemoveField(
            model_name='aggregateresourcecount',
            name='project',
        ),
    ]
