# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-06 15:14
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0002_immutable_default_json'),
        ('marketplace', '0012_offering_default_value'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='offering',
            name='provider',
        ),
        migrations.AddField(
            model_name='offering',
            name='customer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='structure.Customer'),
        ),
    ]
