# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-03 17:43
from __future__ import unicode_literals

from django.db import migrations
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_azure', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='virtualmachine',
            name='private_ips',
            field=waldur_core.core.fields.JSONField(blank=True, default=list, help_text='List of private IP addresses'),
        ),
        migrations.AlterField(
            model_name='virtualmachine',
            name='public_ips',
            field=waldur_core.core.fields.JSONField(blank=True, default=list, help_text='List of public IP addresses'),
        ),
    ]
