# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-08-31 11:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0029_orderitem_state'),
    ]

    operations = [
        migrations.AddField(
            model_name='attribute',
            name='required',
            field=models.BooleanField(default=False, help_text='A value must be provided for the attribute.'),
        ),
    ]
