# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-04-02 07:58
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0073_plan_period_to_usage'),
    ]

    operations = [
        migrations.AddField(
            model_name='attribute',
            name='default',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True),
        ),
    ]
