# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-24 13:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0041_project_resource_count'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='_file',
            field=models.TextField(blank=True, editable=False),
        ),
    ]
