# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-05-07 13:42
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('packages', '0013_packagetemplate_article_code'),
    ]

    operations = [
        migrations.AlterField(
            model_name='openstackpackage',
            name='tenant',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='openstack.Tenant'),
        ),
    ]
