# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-08-22 20:29
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0023_add_billing'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='screenshots',
            new_name='screenshot',
        ),
    ]
