# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-09-03 08:29
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0025_offering_plan_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='offering',
            name='type',
        ),
        migrations.AlterField(
            model_name='offering',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='support.OfferingTemplate'),
        ),
    ]
