# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-12-08 09:34
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('invoices', '0025_servicedowntime'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='_file',
            field=models.TextField(blank=True, editable=False),
        ),
    ]
