# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-08-21 08:55
from __future__ import unicode_literals

from django.db import migrations
import django_fsm


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0020_offering_vendor_details'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='state',
            field=django_fsm.FSMIntegerField(choices=[(1, 'requested for approval'), (2, 'executing'), (3, 'done'), (4, 'terminated'), (5, 'erred'), (6, 'rejected')], default=1),
        ),
    ]
