# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-27 12:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0035_offering_plan'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='offeringtemplate',
            options={'ordering': ['sort_order', 'name']},
        ),
        migrations.AddField(
            model_name='offeringtemplate',
            name='sort_order',
            field=models.PositiveSmallIntegerField(default=1),
        ),
    ]
