# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-12-28 13:46
from __future__ import unicode_literals

from django.db import migrations, models
import django_fsm


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0032_requesttype'),
    ]

    operations = [
        migrations.CreateModel(
            name='IssueStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, help_text='Status name in Jira.', unique=True)),
                ('type', django_fsm.FSMIntegerField(choices=[(0, 'Resolved'), (1, 'Canceled')], default=0)),
            ],
            options={
                'verbose_name': 'Issue status',
                'verbose_name_plural': 'Issue statuses',
            },
        ),
    ]
